@@ -3,9 +3,10 @@ from __future__ import absolute_import
 import datetime
 import math
 import operator
+import uuid
 import warnings
 
-import lxml.builder
+from lxml.builder import E
 import lxml.etree
 
 from .dates import datetime_from_w3_datestring
@@ -20,9 +21,6 @@ except ImportError:
     pytz = None
 
 
-E = lxml.builder.ElementMaker()
-
-
 class SolrError(Exception):
     pass
 
@@ -134,7 +132,8 @@ class SolrField(object):
             elif self.name.endswith("*"):
                 self.wildcard_at_start = False
             else:
-                raise SolrError("Dynamic fields must have * at start or end of name")
+                raise SolrError("Dynamic fields must have * at start or end of name (field %s)" % 
+                        self.name)
 
     def match(self, name):
         if self.dynamic:
@@ -176,7 +175,8 @@ class SolrUnicodeField(SolrField):
         try:
             return unicode(value)
         except UnicodeError:
-            raise SolrError("%s could not be coerced to unicode" % value)
+            raise SolrError("%s could not be coerced to unicode (field %s)" % 
+                    (value, self.name))
 
 
 class SolrBooleanField(SolrField):
@@ -190,7 +190,8 @@ class SolrBooleanField(SolrField):
             elif value.lower() == "false":
                 return False
             else:
-                raise ValueError("sorry, I only understand simple boolean strings")
+                raise ValueError("sorry, I only understand simple boolean strings (field %s)" % 
+                        self.name)
         return bool(value)
 
 
@@ -199,7 +200,8 @@ class SolrBinaryField(SolrField):
         try:
             return str(value)
         except (TypeError, ValueError):
-            raise SolrError("Could not convert data to binary string")
+            raise SolrError("Could not convert data to binary string (field %s)" % 
+                    self.name)
 
     def to_solr(self, value):
         return unicode(value.encode('base64'))
@@ -213,9 +215,11 @@ class SolrNumericalField(SolrField):
         try:
             v = self.base_type(value)
         except (OverflowError, TypeError, ValueError):
-            raise SolrError("%s is invalid value for %s" % (value, self.__class__))
+            raise SolrError("%s is invalid value for %s (field %s)" % 
+                    (value, self.__class__, self.name))
         if v < self.min or v > self.max:
-            raise SolrError("%s out of range for a %s" % (value, self.__class__))
+            raise SolrError("%s out of range for a %s (field %s)" %
+                    (value, self.__class__, self.name))
         return v
 
 
@@ -262,6 +266,25 @@ class SolrRandomField(SolrField):
         raise TypeError("Don't try and store or index values in a RandomSortField")
 
 
+class SolrUUIDField(SolrUnicodeField):
+    def from_solr(self, v):
+        return uuid.UUID(v)
+
+    def from_user_data(self, v):
+        if v == 'NEW':
+            return v
+        elif isinstance(v, uuid.UUID):
+            return v
+        else:
+            return uuid.UUID(v)
+
+    def to_solr(self, v):
+        if v == 'NEW':
+            return v
+        else:
+            return v.urn[9:]
+
+
 class SolrPointField(SolrField):
     def __init__(self, **kwargs):
         super(SolrPointField, self).__init__(**kwargs)
@@ -282,7 +305,17 @@ class SolrPoint2Field(SolrPointField):
 def SolrFieldTypeFactory(cls, name, **kwargs):
     atts = {'stored':True, 'indexed':True}
     atts.update(kwargs)
-    return type('SolrFieldType_%s' % name, (cls,), atts)
+    # This next because otherwise the class names aren't globally
+    # visible or useful, which is confusing for debugging.
+    # We give the new class a name which uniquely identifies it
+    # (but we don't need Solr class, because we've got the same
+    # information in cls anyway.
+    name = 'SolrFieldType_%s_%s' % (cls.__name__, '_'.join('%s_%s' % kv for kv in sorted(atts.items()) if kv[0] != 'class'))
+    # and its safe to put in globals(), because the class is
+    # defined by the constituents of its name.
+    if name not in globals():
+        globals()[name] = type(name, (cls,), atts)
+    return globals()[name]
 
 
 class SolrFieldInstance(object):
@@ -348,6 +381,7 @@ class SolrSchema(object):
         'solr.DateField':SolrDateField,
         'solr.TrieDateField':SolrDateField,
         'solr.RandomSortField':SolrRandomField,
+        'solr.UUIDField':SolrUUIDField,
         'solr.BinaryField':SolrBinaryField,
         'solr.PointType':SolrPointField,
         'solr.LatLonType':SolrPoint2Field,
@@ -482,7 +516,7 @@ class SolrSchema(object):
             values = [self.parse_result_doc(n, name) for n in doc.getchildren()]
             return name, tuple(v[1] for v in values)
         if doc.tag in 'doc':
-            return dict(self.parse_result_doc(n) for n in doc.getchildren())
+            return dict([self.parse_result_doc(n) for n in doc.getchildren()])
         field_class = self.match_field(name)
         if field_class is None and name == "score":
             field_class = SolrScoreField()
@@ -605,6 +639,7 @@ class SolrFacetCounts(object):
 class SolrResponse(object):
     def __init__(self, schema, xmlmsg):
         self.schema = schema
+        self.original_xml = xmlmsg
         doc = lxml.etree.fromstring(xmlmsg)
         details = dict(value_from_node(n) for n in
                        doc.xpath("/response/lst[@name!='moreLikeThis']"))
@@ -629,6 +664,14 @@ class SolrResponse(object):
         else:
             self.more_like_this = None
 
+        # can be computed by MoreLikeThisHandler
+        termsNodes = doc.xpath("/response/*[@name='interestingTerms']")
+        if len(termsNodes) == 1:
+            _, value = value_from_node(termsNodes[0])
+        else:
+            value = None
+        self.interesting_terms = value
+
     def __str__(self):
         return str(self.result)
 
